import React from 'react'
import { Link } from 'md-tools-engine-react/lib/router'
import { FormattedMessage } from 'react-intl'
import classNames from 'classnames'
import { version, Select, Row, Col, Icon, Button, Popover } from 'fish'
import * as utils from '../utils'
import PropTypes from 'prop-types'
const Option = Select.Option

export default class Header extends React.Component {
  static contextTypes = {
    router: PropTypes.object.isRequired,
    intl: PropTypes.object.isRequired
  }

  state = {
    menuVisible: false,
    menuMode: 'horizontal'
  }

  componentDidMount() {
    this.context.router.listen(this.handleHideMenu)

    /* eslint-disable global-require */
    require('enquire.js').register('only screen and (min-width: 0) and (max-width: 992px)', {
      match: () => {
        this.setState({ menuMode: 'inline' })
      },
      unmatch: () => {
        this.setState({ menuMode: 'horizontal' })
      }
    })
    /* eslint-enable global-require */
  }

  handleSearch = value => {
    const { intl, router } = this.context
    router.push({ pathname: utils.getLocalizedPathname(`${value}/`, intl.locale === 'zh-CN') })
    document.querySelector('#search-box .fish-select-search__field').blur()
  }

  handleShowMenu = () => {
    this.setState({
      menuVisible: true
    })
  }

  handleHideMenu = () => {
    this.setState({
      menuVisible: false
    })
  }

  onMenuVisibleChange = visible => {
    this.setState({
      menuVisible: visible
    })
  }

  handleSelectFilter = (value, option) => {
    const optionValue = option.props['data-label']
    return optionValue.indexOf(value.toLowerCase()) > -1
  }

  handleLangChange = () => {
    const pathname = this.props.location.pathname
    const currentProtocol = `${location.protocol}//`
    const currentHref = location.href.substr(currentProtocol.length)

    if (utils.isLocalStorageNameSupported()) {
      localStorage.setItem('locale', utils.isZhCN(pathname) ? 'en-US' : 'zh-CN')
    }

    location.href =
      currentProtocol +
      currentHref.replace(location.pathname, utils.getLocalizedPathname(pathname, !utils.isZhCN(pathname)))
  }

  handleVersionChange = url => {
    window.location.href = url
    // const currentUrl = window.location.href
    // const currentPathname = window.location.pathname
    // window.location.href = currentUrl
    //   .replace(window.location.origin, url)
    //   .replace(
    //     currentPathname,
    //     utils.getLocalizedPathname(currentPathname.replace('-cn', ''), utils.isZhCN(currentPathname))
    //   )
  }

  render() {
    const { menuMode, menuVisible } = this.state
    const { location, picked, isFirstScreen, themeConfig } = this.props
    const docVersions = themeConfig.docVersions ? { ...themeConfig.docVersions, [version]: version } : {}
    const versionOptions = Object.keys(docVersions).map(version => (
      <Option value={docVersions[version]} key={version}>
        {version}
      </Option>
    ))
    const components = picked.components
    const module = location.pathname
      .replace(/(^\/|\/$)/g, '')
      .split('/')
      .slice(0, -1)
      .join('/')
    let activeMenuItem = module || 'home'

    if (activeMenuItem === 'components') {
      activeMenuItem = 'docs/react'
    } else if (location.pathname === 'changelog') {
      activeMenuItem = location.pathname + '/'
    }

    const locale = this.context.intl.locale
    const isZhCN = locale === 'zh-CN'
    const excludedSuffix = isZhCN ? 'en-US.md' : 'zh-CN.md'
    const options = components.filter(({ meta }) => !meta.filename.endsWith(excludedSuffix)).map(({ meta }) => {
      const pathSnippet = meta.filename.split('/')[1]
      const url = `/components/${pathSnippet}`
      const subtitle = meta.subtitle
      return (
        <Option value={url} key={url} data-label={`${meta.title.toLowerCase()} ${subtitle || ''}`}>
          <strong>{meta.title}</strong>
          {subtitle && <span className="fish-component-decs">{subtitle}</span>}
        </Option>
      )
    })

    const headerClassName = classNames({
      clearfix: true,
      'home-nav-white': !isFirstScreen
    })

    const menu = [
      <Button
        style={{ display: 'none' }}
        className="lang"
        type="ghost"
        size="small"
        onClick={this.handleLangChange}
        key="lang"
      >
        <FormattedMessage id="app.header.lang" />
      </Button>
    ]
    if (themeConfig.docVersions) {
      menu.push(
        <Select
          key="version"
          className="version"
          size="small"
          dropdownMatchSelectWidth={false}
          defaultValue={version}
          onChange={this.handleVersionChange}
        >
          {versionOptions}
        </Select>
      )
    }
    const searchPlaceholder = `搜索组件...（${options.length}）`
    return (
      <div id="header" className={headerClassName}>
        {menuMode === 'inline' ? (
          <Popover
            overlayClassName="popover-menu"
            placement="bottomRight"
            content={menu}
            trigger="click"
            visible={menuVisible}
            arrowPointAtCenter
            onVisibleChange={this.onMenuVisibleChange}
          >
            <Icon className="nav-phone-icon" type="menu" onClick={this.handleShowMenu} />
          </Popover>
        ) : null}
        <Row>
          <Col lg={4} md={5} sm={24} xs={24}>
            <Link to={utils.getLocalizedPathname('/readme', isZhCN)} id="logo">
              <img alt="logo" src="//cdncs.101.com/v0.1/static/fish/image/ndlogo.png" />
              <span>{themeConfig.projectName}</span>
            </Link>
          </Col>
          <Col lg={20} md={19} sm={0} xs={0}>
            <div id="search-box">
              <Select
                style={{ width: 250 }}
                combobox
                showSearch
                dropdownClassName="component-select"
                placeholder={searchPlaceholder}
                optionLabelProp="data-label"
                filterOption={this.handleSelectFilter}
                onSelect={this.handleSearch}
                getPopupContainer={trigger => trigger.parentNode}
              >
                {options}
              </Select>
            </div>
            {menuMode === 'horizontal' ? menu : null}
          </Col>
        </Row>
      </div>
    )
  }
}
